//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.messages;

import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.messaging.ImportMessage;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Nachricht;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Eine Mahoe-Bank-Nachricht. Dient als Zwischenspeicher von Bank-Nachrichten,
 * bis sie in der Datenbank gespeichert werden.
 * 
 * @author Frank Mnster
 * 
 */
public class MahoeMessage {

	/**
	 * BLZ
	 */
	private String blz;

	/**
	 * Datum
	 */
	private Date date;

	/**
	 * ID
	 */
	private String id; // NOPMD 

	/**
	 * Nachricht
	 */
	private String message;

	/**
	 * Betreff
	 */
	private String subject;

	/**
	 * berprft, ob die Nachricht schon existiert.
	 * 
	 * @param blz
	 * @param date
	 * @param subject
	 * 
	 * @return true, wenn die Nachricht bereits vorhanden ist, false sonst.
	 */
	public static boolean existsMessage(final String blz, final Date date, final String subject) {

		final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm", Locale.GERMAN); //$NON-NLS-1$
		final String date2 = sdf.format(date);
		final String keyToTest = date2 + subject;
		Logger.debug("Key der neuen Nachricht: " + keyToTest); //$NON-NLS-1$
		boolean exist = false;
		try {
			final DBIterator<Nachricht> messages = getBankMessages(blz);
			while (messages.hasNext()) {
				final Nachricht msg = messages.next();
				final String keyFormDb = msg.getMeta("doubleKey", StringUtils.EMPTY); //$NON-NLS-1$
				Logger.debug("Key der gespeicherten Nachricht: " + keyToTest); //$NON-NLS-1$
				if (keyFormDb.equals(keyToTest)) {
					exist = true;
					break;
				}
			}
		} catch (RemoteException e) {
			Logger.error(I18N.tr("AbstractBankMessages.compareFailed"), e); //$NON-NLS-1$
		}
		return exist;
	}

	/**
	 * Liefert die Nachrichtenliste fr eine Bank
	 * 
	 * @param blz
	 *            die Bankleitzahl
	 * @return die Liste mit den Nachrichten
	 * @throws RemoteException
	 */
	private static DBIterator<Nachricht> getBankMessages(final String blz) throws RemoteException {
		final HBCIDBService service = (HBCIDBService) Settings.getDBService();
		final DBIterator<Nachricht> list = service.createList(Nachricht.class);
		list.addFilter("blz=?", blz); //$NON-NLS-1$
		return list;
	}

	/**
	 * Erzeugt eine neue MahoeNachricht
	 * 
	 * @param blz
	 * @param date
	 * @param subject
	 * @param message
	 */
	public MahoeMessage(final String blz, final Date date, final String subject, final String message) {
		this.blz = blz;
		this.date = date;
		this.subject = subject;
		this.message = message;
	}

	/**
	 * @return the blz
	 */
	public String getBlz() {
		return blz;
	}

	/**
	 * @return the date
	 */
	public Date getDate() {
		return date;
	}

	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @return the message
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * @return the subject
	 */
	public String getSubject() {
		return subject;
	}

	/**
	 * Liefert ein Hibiscus Nachricht, ohne
	 * 
	 * @throws RemoteException
	 * @throws ApplicationException
	 */
	public void saveAsNachricht() throws RemoteException, ApplicationException {

		final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm", Locale.GERMAN); //$NON-NLS-1$

		final Nachricht msg = (Nachricht) Settings.getDBService().createObject(Nachricht.class, null);
		msg.setBLZ(blz);
		msg.setNachricht(message);
		msg.setDatum(date);
		msg.store();

		// Speicher doubleKey
		final String doubleKey = sdf.format(date) + subject;
		msg.setMeta("doubleKey", doubleKey); //$NON-NLS-1$

		Application.getMessagingFactory().sendMessage(new ImportMessage(msg));

	}

	/**
	 * @param blz
	 *            the blz to set
	 */
	public void setBlz(final String blz) {
		this.blz = blz;
	}

	/**
	 * @param date
	 *            the date to set
	 */
	public void setDate(final Date date) {
		this.date = date;
	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setId(final String id) { // NOPMD 
		this.id = id;
	}

	/**
	 * @param message
	 *            the message to set
	 */
	public void setMessage(final String message) {
		this.message = message;
	}

	/**
	 * @param subject
	 *            the subject to set
	 */
	public void setSubject(final String subject) {
		this.subject = subject;
	}

}
